clc
echo on
%% Robust Stability and Mu Analysis
%
%This demo shows how to use the Robust Control Toolbox to analyze and
% quantify the robustness of feedback control systems, including modeling
% stability with the |robuststab| function and gain insight into the
% connection with mu analysis and the |mussv| function.
%% System Description
% The plant model P is uncertain,the signal d is disturbances, 
% and the output e must be kept smallfor adequate disturbance rejection.
%
s=zpk('s'); 			% zero-pole format
plant=1/(s-1); 		% plant
W=((s/2+1)/4)/(s/32+1); 	% Uncertainty weighting function
Ws=(0.25*s+0.6)/(s+0.006) 	% Performance weighting function

%% Creating An Uncertain Plant Model
% As before we generate a full 1x1 uncertailty block
delta = ultidyn('delta',[1 1]);
P = plant*(1+W*delta);

%% Creating two Controllers
% For this example we use the following controllers
K1=-10*(0.9*s+1)/s;
K2=-(2.8*s+1)/s;
pause% strike a key to continue
clc
%% Creating a Closed-Loop System with SYSIC
% To build an uncertain model of the closed-loop system, we'll write the
% interconnection equations directly from the block diagram and
% use the "sysic" function to compute the interconnection:
systemnames = 'P K1 Ws';
inputvar = '[d]';
input_to_P = '[K1]';
input_to_Ws = '[d + P]';
input_to_K1 = '[d + P]';
outputvar = '[Ws]';
cleanupsysic = 'yes';
CLoop1 = sysic;
CLoop1.InputName = {'d'};
CLoop1.OutputName = {'e'};
pause% strike a key to continue
clc
%%
% Alternatively, we can use the |icsignal| and |iconnect| functions to
% build the closed-loop model for controller K2:
IC = iconnect;
d = icsignal(1);
e = icsignal(1);
y = icsignal(1);
IC.Input = [d];
IC.Output = [e];
IC.Equation{1} = equate(y,d+P*K2*y);
IC.Equation{2} = equate(e,Ws*y);
CLoop2 = IC.System;
ClosedLoop.InputName = {'d'};
ClosedLoop.OutputName = {'e'};
pause% strike a key to continue
clc
%%
% Pick 5 random samples of the uncertain closed-loop systems
clf
Psample1=usample(CLoop1,5);
Psample2=usample(CLoop2,5);
impulse(Psample1,':b', Psample2,':r',1)
title('Variability of closed-loop response due to model uncertainty')
legend('K1','K2')
pause% strike a key to continue
clc
%% Robust Stability Analysis
% Does the closed-loop system remain stable for all values of |k|, |delta|
% in the ranges specified above? We use |robuststab| to answer this basic
% robustness question:
[stabmarg1,destabunc1,report1,info1] = robuststab(CLoop1);
stabmarg1
%%
% The variable |stabmarg| gives upper and lower bounds on the robust
% stability margin, a measure of how much uncertainty on |delta| the
% feedback loop can tolerate before becoming unstable. For example, a
% margin of 0.8 indicates that as little as 80% of the specified
% uncertainty level can lead to instability. Here the margin is 1.06,
% which means that the closed loop will remain stable for up to 106%
% of the specified uncertainty. The stability margin is
% quite narrow in this example. The |report| function summarizes
% these results:
report1
pause% strike a key to continue
clc
% Repeat that for the second controller K2, and compare the stability
% margins
[stabmarg2,destabunc2,report2,info2] = robuststab(CLoop2);
stabmarg2
report2
pause% strike a key to continue
clc
%% Robust Stability Analysis: Connection with Mu Analysis
% The structured singular value, or mu, is the mathematical tool used
% by |robuststab| to compute the robust stability margin. If you are
% comfortable with structured singular value analysis, you can use
% the |mussv| function directly to compute mu as a function of frequency
% and reproduce the results above. The function |mussv| is the underlying
% engine for all robustness analysis commands.
%
% To use |mussv|, we first extract the |(M,Delta)| decomposition of the
% uncertain closed-loop model |ClosedLoop|, where |Delta| is a
% block-diagonal matrix of (normalized) uncertain elements.
% The 3rd output argument of |lftdata|, |BlkStruct|, describes the block-diagonal
% structure of |Delta| and can be used directly by |mussv|

[M1,Delta1,BlkStruct1] = lftdata(CLoop1);
[M2,Delta2,BlkStruct2] = lftdata(CLoop2);

%%
% For a robust stability analysis, only the channels of |M| associated
% with the uncertainty channels are used. Based on the row/column size of
% |Delta|, select the proper columns and rows of |M|. Remember that the
% rows of |Delta| correspond to the columns of |M|, and vice versa.
% Consequently, the column dimension of |Delta| is used to specify the rows
% of |M|:
M11 = M1(1,1);
N11 = M2(1,1);
pause% strike a key to continue
clc
%%
% Mu-analysis is performed on a finite grid of frequencies. For comparison
% purposes, evaluate the frequency response of |M11| over the same
% frequency grid as used for the |robuststab| analysis.
omega1 = info1.Frequency;
omega2 = info2.Frequency;
M11_g = frd(M11,omega1);
N11_g = frd(N11,omega2);
%%
% Compute |mu(M11)| at these frequencies and plot the resulting lower and
% upper bounds:
mubnds1 = mussv(M11_g,BlkStruct1,'s');
mubnds2 = mussv(N11_g,BlkStruct2,'s');
semilogx(mubnds1,'b', mubnds2,'r', info1.MussvBnds,':y')
xlabel('Frequency (rad/sec)');
ylabel('Mu upper/lower bounds');
title('Mu plot of robust stability margins of two controllers');
%%
% The robust stability margin is the reciprocal of the structured singular
% value. Therefore upper bounds from |mussv| become lower bounds
% on the stability margin.
pause% strike a key to continue
clc
%% Robust Performance Analysis
% The input/output gain of a nominally-stable uncertain system model will
% generally degrade for specific values of its uncertain elements.
% Moreover, the maximum possible degradation increases as the uncertain
% elements are allowed to further deviate from their nominal values.
%
% The simplest route to analyzing the robust performance margin of
% the closed-loop system is direct use of the |robustperf| command.
[perfmarg1,perfmargunc1,rep1,inf1] = robustperf(CLoop1);
[perfmarg2,perfmargunc2,rep2,inf2] = robustperf(CLoop2);
%%
% The |perfmarg| variable has upper and lower bounds on the performance
% margin.
perfmarg1
%%
% The |report| variable summarizes the robust performance analysis.
disp(rep1)
pause% strike a key to continue
%%
% The |report| variable summarizes the robust performance analysis.

disp(rep1)
clc
% Repeat the report for the second controller
perfmarg2
disp(rep2)
disp(rep2)
%%
% Finally, we plot the bounds from |mussv|, which is the
% underlying engine for the robustness analysis. The peak value is the
% reciprocal of the performance margin, and the frequency at which the
% peak occurs is the critical frequency.
semilogx(inf1.MussvBnds,'b',inf2.MussvBnds,'r')
xlabel('Frequency (rad/sec)');
ylabel('Mu upper/lower bounds');
title('Robust Performance Mu Plot');
%
% This concludes this example
% For study the connection of the above robust performance analysis
% with Mu Analysis study the Matlab mu_demo od Robust Control Toolbox.

